@@ -8,7 +8,15 @@ See LICENSES/README.md for more Information
 """
 
 import sys
+import warnings
+
 from opsicommon.logging import logger
+
+if getattr(sys, "frozen", False):
+	# Disable warnings if frozen
+	warnings.simplefilter("ignore", ResourceWarning)
+	warnings.simplefilter("ignore", DeprecationWarning)
+
 from .opsipxeconfdinit import OpsipxeconfdInit
 
 
@@ -22,7 +30,7 @@ def main() -> None:
 		OpsipxeconfdInit()
 	except SystemExit:
 		pass
-	except Exception as err:  # pylint: disable=broad-except
+	except Exception as err:
 		logger.error(err, exc_info=True)
 		print(f"ERROR: {err}", file=sys.stderr)
 		sys.exit(1)
