@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # opsipxeconfd is part of the desktop management solution opsi http://www.opsi.org
-# Copyright (c) 2013-2021 uib GmbH <info@uib.de>
+# Copyright (c) 2013-2023 uib GmbH <info@uib.de>
 # All rights reserved.
 # License: AGPL-3.0
 
@@ -9,44 +9,94 @@
 opsipxeconfd - init
 """
 
-import codecs
 import os
 import sys
 import threading
-import time
-import argparse
-from signal import SIGHUP, SIGINT, SIGTERM, signal
+from argparse import ArgumentDefaultsHelpFormatter, Namespace
 from collections import OrderedDict
-import configargparse
+from contextlib import closing, contextmanager
+from io import StringIO
+from signal import SIGHUP, SIGINT, SIGTERM, signal
+from socket import AF_UNIX, SOCK_STREAM, socket
+from time import sleep
+from types import FrameType
+from typing import Any, Generator
 
-from opsicommon.logging import logger, DEFAULT_FORMAT, LOG_WARNING, log_context, set_filter_from_string
+from configargparse import (  # type: ignore[import]
+	ArgParser,
+	ConfigFileParser,
+	ConfigFileParserException,
+)
+from opsicommon import __version__ as python_opsi_common_version
+from opsicommon.logging import (
+	DEFAULT_FORMAT,
+	LOG_WARNING,
+	get_logger,
+	log_context,
+	set_filter_from_string,
+)
+from opsicommon.types import forceInt, forceUnicode, forceUnicodeList
 
-from OPSI.Backend.OpsiPXEConfd import ServerConnection
-from OPSI.Util import getfqdn
-from OPSI.Types import forceHostId, forceUnicodeList
-from OPSI import __version__ as python_opsi_version
-
-from .logging import init_logging
-from .setup import setup
-from .util import temporaryPidFile
-from .opsipxeconfd import Opsipxeconfd
 from . import __version__
+from .logging import init_logging
+from .opsipxeconfd import Opsipxeconfd, opsi_config
+from .setup import setup
+from .util import pid_file
 
 DEFAULT_CONFIG_FILE = "/etc/opsi/opsipxeconfd.conf"
+ERROR_MARKER = "(ERROR)"
 
 
-class OpsipxeconfdConfigFileParser(configargparse.ConfigFileParser):  # pylint: disable=abstract-method
-	def get_syntax_description(self):
+logger = get_logger()
+
+
+@contextmanager
+def create_unix_socket(port: str, timeout: float = 5.0) -> Generator[socket, None, None]:
+	logger.notice("Creating unix socket %s", port)
+	_socket = socket(AF_UNIX, SOCK_STREAM)
+	_socket.settimeout(timeout)
+	try:
+		with closing(_socket) as unix_socket:
+			unix_socket.connect(port)
+			yield unix_socket
+	except Exception as err:
+		raise RuntimeError(f"Failed to connect to socket '{port}': {err}") from err
+
+
+class ServerConnection:
+	def __init__(self, port: str, timeout: float = 10.0) -> None:
+		self.port = port
+		self.timeout = forceInt(timeout)
+
+	def send_command(self, cmd: str) -> str:
+		with create_unix_socket(self.port, timeout=self.timeout) as unix_socket:
+			unix_socket.send(forceUnicode(cmd).encode("utf-8"))
+			result = ""
+			try:
+				for part in iter(lambda: unix_socket.recv(4096), b""):
+					logger.trace("Received %s", part)
+					result += forceUnicode(part)
+			except Exception as err:
+				raise RuntimeError(f"Failed to receive: {err}") from err
+
+		if result.startswith(ERROR_MARKER):
+			raise RuntimeError(f"Command '{cmd}' failed: {result}")
+
+		return result
+
+
+class OpsipxeconfdConfigFileParser(ConfigFileParser):
+	def get_syntax_description(self) -> str:
 		return ""
 
-	def parse(self, stream):  # pylint: disable=too-many-branches
+	def parse(self, stream: StringIO) -> dict[str, Any]:
 		items = OrderedDict()
 		for i, line in enumerate(stream):
 			line = line.strip()
 			if not line or line.startswith(("#", ";", "/")):
 				continue
 			if "=" not in line:
-				raise configargparse.ConfigFileParserException(f"Unexpected line {i} in {getattr(stream, 'name', 'stream')}: {line}")
+				raise ConfigFileParserException(f"Unexpected line {i} in {getattr(stream, 'name', 'stream')}: {line}")
 
 			(option, value) = line.split("=", 1)
 			option = option.strip()
@@ -72,10 +122,6 @@ class OpsipxeconfdConfigFileParser(configargparse.ConfigFileParser):  # pylint:
 				items["pxe-dir"] = value
 			elif option == "pxe config template":
 				items["pxe-conf-template"] = value
-			elif option == "uefi netboot config template x86":
-				items["uefi-conf-template-x86"] = value
-			elif option == "uefi netboot config template x64":
-				items["uefi-conf-template-x64"] = value
 			elif option == "max pxe config writers":
 				items["max-pxe-config-writers"] = value
 			elif option == "max control connections":
@@ -85,13 +131,11 @@ class OpsipxeconfdConfigFileParser(configargparse.ConfigFileParser):  # pylint:
 			elif option == "dispatch config file":
 				items["dispatch-config-file"] = value
 			else:
-				raise configargparse.ConfigFileParserException(
-					f"Unexpected option in line {i} in {getattr(stream, 'name', 'stream')}: {option}"
-				)
+				raise ConfigFileParserException(f"Unexpected option in line {i} in {getattr(stream, 'name', 'stream')}: {option}")
 		return items
 
 
-def parse_args() -> argparse.Namespace:
+def parse_args(parse_config_file: bool = True) -> Namespace:
 	"""
 	Parses command line arguments.
 
@@ -99,16 +143,16 @@ def parse_args() -> argparse.Namespace:
 	to extract keywords and values to set for certain variables.
 
 	:returns: Namespace object containing all provided settings (or defaults).
-	:rtype: argparse.Namespace
+	:rtype: Namespace
 	"""
-	parser = configargparse.ArgParser(
+	parser = ArgParser(
 		config_file_parser_class=OpsipxeconfdConfigFileParser,
-		formatter_class=lambda prog: argparse.ArgumentDefaultsHelpFormatter(prog, max_help_position=30, width=100),
+		formatter_class=lambda prog: ArgumentDefaultsHelpFormatter(prog, max_help_position=30, width=100),
 	)
 	parser.add("--version", "-v", help="Show version information and exit.", action="store_true")
 	parser.add("--no-fork", "-F", dest="nofork", help="Do not fork to background.", action="store_true")
-	parser.add("-c", "--conffile", required=False, is_config_file=True, default=DEFAULT_CONFIG_FILE, help="Path to config file.")
-	parser.add("--setup", action="store_true", help="Set up the environment and exit.")
+	if parse_config_file:
+		parser.add("-c", "--conffile", required=False, is_config_file=True, default=DEFAULT_CONFIG_FILE, help="Path to config file.")
 	parser.add(
 		"--log-level",
 		"--loglevel",
@@ -201,29 +245,15 @@ def parse_args() -> argparse.Namespace:
 		"--pxe-dir",
 		dest="pxeDir",
 		env_var="OPSIPXECONFD_PXE_DIR",
-		default="/tftpboot/linux/pxelinux.cfg",
+		default="/tftpboot/opsi/opsi-linux-bootimage/cfg",
 		help="Location of the pxe directory.",
 	)
 	parser.add(
 		"--pxe-conf-template",
 		dest="pxeConfTemplate",
 		env_var="OPSIPXECONFD_PXE_CONF_TEMPLATE",
-		default="/tftpboot/linux/pxelinux.cfg/install",
-		help="Location of the pxe config template.",
-	)
-	parser.add(
-		"--uefi-conf-template-x86",
-		dest="uefiConfTemplateX86",
-		env_var="OPSIPXECONFD_UEFI_CONF_TEMPLATE_X86",
-		default="/tftpboot/linux/pxelinux.cfg/install-elilo-x86",
-		help="Location of the uefi x86 config template.",
-	)
-	parser.add(
-		"--uefi-conf-template-x64",
-		dest="uefiConfTemplateX64",
-		env_var="OPSIPXECONFD_UEFI_CONF_TEMPLATE_X64",
-		default="/tftpboot/linux/pxelinux.cfg/install-grub-x64",
-		help="Location of the uefi x64 config template.",
+		default="/tftpboot/opsi/opsi-linux-bootimage/cfg/install-grub-x64",
+		help="Location of the pxe/uefi/grub config template.",
 	)
 	parser.add(
 		"--max-connections",
@@ -244,26 +274,27 @@ def parse_args() -> argparse.Namespace:
 	parser.add(
 		"command",
 		nargs="?",
-		choices=("start", "stop", "status", "update"),
-		metavar="<command>",
+		choices=("start", "stop", "status", "update", "setup"),
+		metavar="COMMAND",
+		help="The COMMAND to perform (start/stop/status/update/setup).",
 	)
 
 	opts = parser.parse_args()
 
 	if opts.version:
-		print(f"{__version__} [python-opsi={python_opsi_version}]")
+		print(f"{__version__} [python-opsi-common={python_opsi_common_version}]")
 		sys.exit(0)
 
-	has_command = opts.command and (opts.command in ["start", "stop", "update", "status"])
+	has_command = opts.command and (opts.command in ["start", "stop", "update", "status", "setup"])
 
-	if not opts.setup and not has_command:
+	if not has_command:
 		parser.print_help()
 		sys.exit(1)
 
 	return opts
 
 
-def assemble_command(config):
+def assemble_command(config: dict[str, Any]) -> list[str]:
 	command = [config["command"]]
 	if config.get("nofork"):
 		command.append("--no-fork")
@@ -293,16 +324,22 @@ class OpsipxeconfdInit:
 
 		This constructor creates an OpsipxeconfdInit instance. The settings are
 		determined by values in the config file and command line arguments
-		(previously parsed and converted into an argparse.Namespace).
+		(previously parsed and converted into an Namespace).
 		Depending on the command specified on command line, an action is triggered.
 		This could be start, stop, update or status.
 
-		:param opts: Parsed command line arguments as argparse.Namespace.
-		:type opts: argparse.Namespace.
+		:param opts: Parsed command line arguments as Namespace.
+		:type opts: Namespace.
 		"""
+		# First parse to handle --help and --version
+		parse_args(parse_config_file=False)
+		self.update_config_file()
 		self.config = vars(parse_args())
+
+		opsi_config._upgrade_config = True
+
 		self.config["port"] = "/var/run/opsipxeconfd/opsipxeconfd.socket"
-		self.config["depotId"] = forceHostId(getfqdn())
+		self.config["depotId"] = opsi_config.get("host", "id")
 		self.config["daemon"] = True
 		if self.config["nofork"] and self.config["command"] == "start":
 			self.config["daemon"] = False
@@ -313,40 +350,45 @@ class OpsipxeconfdInit:
 		os.umask(0o077)
 		self._pid = 0
 
-		self.updateConfigFile()
-
 		if self.config.get("logFilter"):
 			set_filter_from_string(self.config["logFilter"])
 		init_logging(self.config)
-		if self.config.get("setup"):
+
+		if self.config.get("command") == "setup":
 			with log_context({"instance": "Opsipxeconfd setup"}):
 				setup(self.config)
 			return  # TODO: exit code handling
 
 		if self.config.get("command") == "start":
-			# Call signalHandler on signal SIGHUP, SIGTERM, SIGINT
-			signal(SIGHUP, self.signalHandler)
-			signal(SIGTERM, self.signalHandler)
-			signal(SIGINT, self.signalHandler)
+			# Call signal_handler on signal SIGHUP, SIGTERM, SIGINT
+			signal(SIGHUP, self.signal_handler)
+			signal(SIGTERM, self.signal_handler)
+			signal(SIGINT, self.signal_handler)
 
 			if self.config["daemon"]:
 				self.daemonize()
+
 			with log_context({"instance": "Opsipxeconfd start"}):
-				with temporaryPidFile(self.config["pidFile"]):
+				with pid_file(self.config["pidFile"]):
+					setup(self.config)
 					self._opsipxeconfd = Opsipxeconfd(self.config)
 					self._opsipxeconfd.start()
-					time.sleep(3)
-					while self._opsipxeconfd.isRunning():
-						time.sleep(1)
+					sleep(1)
+					while self._opsipxeconfd.is_running():
+						sleep(1)
 					self._opsipxeconfd.join(30)
+					if self._opsipxeconfd.error:
+						print(f"ERROR: {self._opsipxeconfd.error}")
+						sys.exit(1)
+					sys.exit(0)
 		else:
 			with log_context({"instance": " ".join(["Opsipxeconfd", self.config["command"]])}):
 				command = assemble_command(self.config)
 				con = ServerConnection(self.config["port"], timeout=5.0)
-				result = con.sendCommand(" ".join(forceUnicodeList(command)))
+				result = con.send_command(" ".join(forceUnicodeList(command)))
 				print(result)
 
-	def signalHandler(self, signo, stackFrame) -> None:  # pylint: disable=unused-argument
+	def signal_handler(self, signo: int, frame: FrameType | None) -> None:
 		"""
 		Signal Handler for OpsipxeconfdInit.
 
@@ -367,7 +409,7 @@ class OpsipxeconfdInit:
 		if signo == SIGHUP:
 			try:
 				self.config.update(vars(parse_args()))
-				self._opsipxeconfd.setConfig(self.config)
+				self._opsipxeconfd.set_config(self.config)
 				self._opsipxeconfd.reload()
 			except AttributeError:
 				pass  # probably set to None
@@ -380,27 +422,55 @@ class OpsipxeconfdInit:
 		for thread in threading.enumerate():
 			logger.debug("Running thread after signal: %s", thread)
 
-	def updateConfigFile(self) -> None:
+	def update_config_file(self) -> None:
 		"""
 		Updates Opsipxeconfd config file.
 
 		This method modifies the data written in the configFile to conform to
 		the standard logging format.
 		"""
-		with codecs.open(self.config["conffile"], "r", "utf-8") as file:
-			data = file.read()
-		new_data = data.replace("[%l] [%D] %M (%F|%N)", DEFAULT_FORMAT)
-		new_data = new_data.replace("%D", "%(asctime)s")
-		new_data = new_data.replace("%T", "%(thread)d")
-		new_data = new_data.replace("%l", "%(opsilevel)d")
-		new_data = new_data.replace("%L", "%(levelname)s")
-		new_data = new_data.replace("%M", "%(message)s")
-		new_data = new_data.replace("%F", "%(filename)s")
-		new_data = new_data.replace("%N", "%(lineno)s")
-		if new_data != data:
-			logger.notice("Updating config file: %s", self.config["conffile"])
-			with codecs.open(self.config["conffile"], "w", "utf-8") as file:
-				file.write(new_data)
+		config_file = DEFAULT_CONFIG_FILE
+		if not os.path.exists(config_file):
+			return
+
+		default_tftp_root = tftp_root = "/tftpboot"
+		if not os.path.exists(tftp_root) and os.path.exists("/var/lib/tftpboot"):
+			tftp_root = "/var/lib/tftpboot"
+
+		lines = []
+		changed = False
+		with open(config_file, encoding="utf-8") as file:
+			for line in file.readlines():
+				cur_line = line
+
+				if line.startswith("uefi netboot config template"):
+					line = f"#{line}"
+				elif line.startswith("pxe config dir"):
+					line = line.replace("/linux/pxelinux.cfg", "/opsi/opsi-linux-bootimage/cfg")
+					line = line.replace("/opsi/pxelinux.cfg", "/opsi/opsi-linux-bootimage/cfg")
+					line = line.replace(f" {default_tftp_root}/opsi", f" {tftp_root}/opsi")
+				elif line.startswith("pxe config template"):
+					line = line.replace("/linux/pxelinux.cfg/install", "/opsi/opsi-linux-bootimage/cfg/install-grub-x64")
+					line = line.replace("/opsi/pxelinux.cfg/install", "/opsi/opsi-linux-bootimage/cfg/install-grub-x64")
+					line = line.replace(f" {default_tftp_root}/opsi", f" {tftp_root}/opsi")
+				elif line.startswith("log format"):
+					line = line.replace("[%l] [%D] %M (%F|%N)", DEFAULT_FORMAT)
+					line = line.replace("%D", "%(asctime)s")
+					line = line.replace("%T", "%(thread)d")
+					line = line.replace("%l", "%(opsilevel)d")
+					line = line.replace("%L", "%(levelname)s")
+					line = line.replace("%M", "%(message)s")
+					line = line.replace("%F", "%(filename)s")
+					line = line.replace("%N", "%(lineno)s")
+
+				lines.append(line)
+				if not changed and line != cur_line:
+					changed = True
+
+		if changed:
+			logger.notice("Updating config file: %s", config_file)
+			with open(config_file, "w", encoding="utf-8") as file:
+				file.writelines(lines)
 
 	def daemonize(self) -> None:
 		"""
@@ -416,7 +486,7 @@ class OpsipxeconfdInit:
 				# Parent exits
 				sys.exit(0)
 		except OSError as err:
-			raise Exception(f"First fork failed: {err}") from err  # pylint: disable=broad-exception-raised
+			raise RuntimeError(f"First fork failed: {err}") from err
 
 		# Do not hinder umounts
 		os.chdir("/")
@@ -429,7 +499,7 @@ class OpsipxeconfdInit:
 			if self._pid > 0:
 				sys.exit(0)
 		except OSError as err:
-			raise Exception(f"Second fork failed: {err}") from err  # pylint: disable=broad-exception-raised
+			raise RuntimeError(f"Second fork failed: {err}") from err
 
 		# Close standard output and standard error.
 		os.close(0)
